import random

from . import constants
from . import exceptions


class PasswordGenerator:
    def __init__(self, length=False, ignore=False, only=False, include=False, repeat=False, separator=False,
                 separator_length=False, separation=False):
        self.__password = None
        self.__available_char = None
        self.__possibility = None
        self.__error = None
        self.length = length
        self.ignore = ignore
        self.only = only
        self.include = include
        self.repeat = repeat
        self.separator_length = separator_length
        self.separator = separator if separator else separation

    def __all_possible_chars(self):
        self.__possibility = constants.POSSIBLE_UPPERCASE_CHARS + constants.POSSIBLE_LOWERCASE_CHARS + constants.POSSIBLE_NUMBERS + constants.POSSIBLE_SPECIAL_CHARS
        self.__available_char = {"alphabets": constants.POSSIBLE_UPPERCASE_CHARS + constants.POSSIBLE_LOWERCASE_CHARS,
                                 "uppercase": constants.POSSIBLE_UPPERCASE_CHARS,
                                 "lowercase": constants.POSSIBLE_LOWERCASE_CHARS,
                                 "numbers": constants.POSSIBLE_NUMBERS,
                                 "symbols": constants.POSSIBLE_SPECIAL_CHARS
                                 }

    def __set_length(self):
        if not self.length:
            if not self.only:
                self.length = random.randint(constants.DEFAULT_MIN_PASS_LEN, constants.DEFAULT_MAX_PASS_LEN)
            else:
                self.__error = ValueError('[-] Password length must be given.')

    def __add_only_wanted(self):
        possibility = constants.EMPTY_STRING
        try:
            choices = [character for character in self.only.split(',')]
            for choice in choices:
                if choice.lower() in self.__available_char:
                    possibility += self.__available_char[choice.lower()]
                if not choice.lower() in self.__available_char:
                    for char in choice:
                        possibility += char
        except Exception:
            raise exceptions.GenpasswdException
        self.__possibility = possibility

    def __remove_unwanted(self):
        try:
            choices = [characters for characters in self.ignore.split(',')]
            if self.ignore == ',' or ',,,' in self.ignore or ',,' in self.ignore or len(choices) > 0:
                possibility = self.__possibility.replace(',', constants.EMPTY_STRING)
                for choice in choices:
                    if choice.lower() in self.__available_char:
                        for char in self.__available_char[choice.lower()]:
                            possibility = possibility.replace(char, constants.EMPTY_STRING)
                    if choice not in self.__available_char:
                        for char in choice:
                            possibility = possibility.replace(char, constants.EMPTY_STRING)
                self.__possibility = possibility
        except Exception:
            raise exceptions.GenpasswdException

    def __include_characters(self):
        possibility = self.__possibility
        try:
            choices = [character for character in self.include.split(',')]
            for choice in choices:
                if choice.lower() in self.__available_char:
                    for char in self.__available_char[choice.lower()]:
                        if char not in possibility:
                            possibility += char
                if not choice.lower() in self.__available_char:
                    for char in choice:
                        if char not in possibility:
                            possibility += char
        except Exception:
            raise exceptions.GenpasswdException
        self.__possibility = possibility

    def __repeat_char(self):
        if self.repeat is None or self.repeat is True:
            self.__possibility *= self.length

    def __check(self):
        if self.length and ((self.length > len(
                self.__possibility)) or self.length > constants.PASSWORD_LENGTH_LIMIT) and self.__error is None:
            self.__error = ValueError('[-] Password length must be less.')

    def __separated_pass(self):
        if type(self.separator) is bool:
            self.separator = constants.DEFAULT_SEPARATOR
        if not self.separator_length:
            self.separator_length = constants.DEFAULT_SEPARATE_LENGTH
        final_password = constants.EMPTY_STRING
        for i in range(len(self.__password)):
            final_password += self.separator + self.__password[i] if i % self.separator_length == 0 and i != 0 else \
                self.__password[i]
        return final_password

    def __filter(self):
        if self.only:
            self.__add_only_wanted()
        if self.include:
            self.__include_characters()
        if self.ignore:
            self.__remove_unwanted()
        self.__repeat_char()

    def generate(self):
        self.__all_possible_chars()
        self.__set_length()
        self.__filter()
        self.__check()
        if self.__error is not None:
            return self.__error
        self.__password = constants.EMPTY_STRING.join(random.sample(self.__possibility, self.length))
        return self.__separated_pass() if self.separator or self.separator_length else self.__password



import argparse

from . import genpasswd


# def usage_string():
#     return """{0}
# {2:{1}} [--length] [--ignore]
# {2:{1}} [--include] [--only] [--repeat]
# {2:{1}} [--separation] [--separator] [--seplen]
# {2:{1}} --help --version""".format('genpasswd', len('usage:'), '')

def get_argument():
    parser = argparse.ArgumentParser(usage="genpasswd [options]")
    parser.add_argument('-v', '--version', action='version', help='show version number and exit.', version="1.3.5")
    group = parser.add_argument_group("to customize Password")
    group.add_argument("-l", "--length", dest="length", type=int, metavar='', help="to set length to the password")
    group.add_argument("-n", "--ignore", dest="ignore", metavar='',
                       help="to ignore unwanted characters to the password")
    group.add_argument("-i", "--include", dest="include", metavar='', help="to include characters to the password")
    group.add_argument("-o", "--only", dest="only", metavar='', help="to create password only using wanted characters")
    group.add_argument("-s", "--separator", dest="separator", metavar='', help="the separator character")
    group.add_argument("-c", "--seplen", dest="separatorlength", type=int, metavar='',
                       help="the length of characters between separator")
    group.add_argument("--repeat", dest="repeat", action='store_true', default=False,
                       help="to repeat the characters in the password (default : %(default)s)")
    group.add_argument("--separation", dest="separation", default=False, action="store_true",
                       help="to separate password characters using separator (default : %(default)s)")
    parser.add_argument_group(group)
    options = parser.parse_args()
    return options


def generate_password(rep, separation, pass_len=False, wanted=False, ign=False, inc=False, sep=False, sep_len=False):
    arg = genpasswd.PasswordGenerator(length=pass_len, only=wanted, ignore=ign, include=inc, repeat=rep, separator=sep,
                                      separator_length=sep_len, separation=separation)
    password = arg.generate()
    return password


def main():
    options = get_argument()
    password = generate_password(options.repeat, options.separation, options.length, options.only, options.ignore,
                                 options.include, options.separator, options.separatorlength)
    print(f"\n{password}")


if __name__ == "__main__":
    main()
